// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include "sim.h"
#include "dump.h"

static int unused __attribute__ ((unused));
static int dump_number;
void dump_init(struct device *in)
{
	dump_number = 0;
	set_dump_status(dump_lock, FALSE);
}

void dump_write_to_disk(struct device *in)
{
	char postfix[100];
	char snapshots_dir[200];
	char out_dir[200];
	int dumped = FALSE;
	struct stat st = { 0 };

	sprintf(postfix, "%d", dump_number);

	join_path(2, snapshots_dir, in->outputpath, "snapshots");

	if (stat(snapshots_dir, &st) == -1) {
#else
		mkdir(snapshots_dir, 0700);
#endif
	}

	join_path(2, out_dir, snapshots_dir, postfix);

	if (stat(out_dir, &st) == -1) {
#else
		mkdir(out_dir, 0700);
#endif
	}

	if (get_dump_status(dump_1d_slices) == TRUE) {
		dump_1d_slice(in, "snapshots", postfix);
		dumped = TRUE;
	}

	if (get_dump_status(dump_energy_slice_switch) == TRUE) {
		dump_energy_slice(in, dump_number, in->dump_slicepos);
		dumped = TRUE;
	}

	if (get_dump_status(dump_pl) == TRUE) {
		exp_cal_emission(dump_number, in);
		dumped = TRUE;
	}

	if (dumped == TRUE) {
		dump_number++;
	}

}
