// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <util.h>
#include <true_false.h>
#include "patch.h"
#include "../../dump_ctrl.h"
#include "../../inp.h"

static int unused __attribute__ ((unused));

void patch(char *dest, char *patch_file)
{
	FILE *in;
	char token[100];
	char file[100];
	char newtext[100];

	if ((in = fopen(patch_file, "r")) == NULL) {
		ewe("Error opening file: %s\n", patch_file);
	}
	char filetoedit[200];
	if (get_dump_status(dump_iodump) == TRUE)
		printf("Patch %s\n", patch_file);
	int found = FALSE;

	struct inp_file ifile;
	inp_init(&ifile);

	do {
		unused = fscanf(in, "%s", token);

		if (strcmp(token, "#end") == 0) {
			break;
		}
		if (token[0] != '#') {
			ewe("error token does not begin with #\n", token);
		} else {
			found = TRUE;
			unused = fscanf(in, "%s", file);
			unused = fscanf(in, "%s", newtext);
			join_path(2, filetoedit, dest, file);
			inp_load(&ifile, filetoedit);
			inp_replace(&ifile, token, newtext);

		}

	} while (!feof(in));

	inp_free(&ifile);

	if (strcmp(token, "#end") != 0) {
		ewe("Error at end of patch file\n");
	}

	fclose(in);

	if (found == FALSE) {
		ewe("Token not found\n");
	}

	return;
}
